package javax.smartcardio;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.security.Permission;

public class CardPermission extends Permission
{
    private static final long serialVersionUID = 7146787880530705613L;
    private static final int A_CONNECT = 1;
    private static final int A_EXCLUSIVE = 2;
    private static final int A_GET_BASIC_CHANNEL = 4;
    private static final int A_OPEN_LOGICAL_CHANNEL = 8;
    private static final int A_RESET = 16;
    private static final int A_TRANSMIT_CONTROL = 32;
    private static final int A_ALL = 63;
    private static final int[] ARRAY_MASKS;
    private static final String S_CONNECT = "connect";
    private static final String S_EXCLUSIVE = "exclusive";
    private static final String S_GET_BASIC_CHANNEL = "getBasicChannel";
    private static final String S_OPEN_LOGICAL_CHANNEL = "openLogicalChannel";
    private static final String S_RESET = "reset";
    private static final String S_TRANSMIT_CONTROL = "transmitControl";
    private static final String S_ALL = "*";
    private static final String[] ARRAY_STRINGS;
    private transient int mask;
    private volatile String actions;
    
    public CardPermission(String name, String s) {
        super(name);
        if (name == null) {
            throw new NullPointerException();
        }
        this.mask = getMask(s);
        return;
    }
    
    private static int getMask(String str) {
        if (str == null || str.length() == 0) {
            throw new IllegalArgumentException("actions must not be empty");
        }
        int i = 0;
        while (i < CardPermission.ARRAY_STRINGS.length) {
            if (str == CardPermission.ARRAY_STRINGS[i]) {
                return CardPermission.ARRAY_MASKS[i];
            }
            ++i;
        }
        if (str.endsWith(",")) {
            throw new IllegalArgumentException("Invalid actions: '" + str + "'");
        }
        int n = 0;
        String[] split = str.split(",");
        int length = split.length;
        int j = 0;
    Label_0111:
        while (j < length) {
            String s = split[j];
            int k = 0;
            while (k < CardPermission.ARRAY_STRINGS.length) {
                if (CardPermission.ARRAY_STRINGS[k].equalsIgnoreCase(s)) {
                    n |= CardPermission.ARRAY_MASKS[k];
                    ++j;
                    continue Label_0111;
                }
                ++k;
            }
            throw new IllegalArgumentException("Invalid action: '" + s + "'");
        }
        return n;
    }
    
    private static String getActions(int n) {
        if (n == 63) {
            return "*";
        }
        int n2 = 1;
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < CardPermission.ARRAY_MASKS.length; ++i) {
            int n3 = CardPermission.ARRAY_MASKS[i];
            if ((n & n3) == n3) {
                if (n2 == 0) {
                    sb.append(",");
                }
                else {
                    n2 = 0;
                }
                sb.append(CardPermission.ARRAY_STRINGS[i]);
            }
        }
        return sb.toString();
    }
    
    @Override
    public String getActions() {
        if (this.actions == null) {
            this.actions = getActions(this.mask);
        }
        return this.actions;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof CardPermission)) {
            return false;
        }
        CardPermission cardPermission = (CardPermission)permission;
        if ((this.mask & cardPermission.mask) != cardPermission.mask) {
            return false;
        }
        String name = this.getName();
        return name.equals("*") || name.equals(cardPermission.getName());
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CardPermission)) {
            return false;
        }
        CardPermission cardPermission = (CardPermission)o;
        return this.getName().equals(cardPermission.getName()) && this.mask == cardPermission.mask;
    }
    
    @Override
    public int hashCode() {
        return this.getName().hashCode() + 31 * this.mask;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (this.actions == null) {
            this.getActions();
        }
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.mask = getMask(this.actions);
    }
    
    static {
        ARRAY_MASKS = new int[] { 63, 1, 2, 4, 8, 16, 32 };
        ARRAY_STRINGS = new String[] { "*", "connect", "exclusive", "getBasicChannel", "openLogicalChannel", "reset", "transmitControl" };
    }
}
